/*
 * FT-857-Talk
 * -----------
 * Arduino UNO, Arduino IDE 1.8.15, talkie-Lib, FT857D-Lib
 * Sprachausgabe an Arduino UNO Pin 3
 * Taster an Arduino UNO Pin 8
 * Serielle Ausgabe TX in Richtung TRX: Arduino UNO Pin 11
 * Einstellung FT857: Menü 20, CAT, Menü 19, CAT Rate 9600 
 * Serielle Eingabe von TRX in Richtung Arduino UNO Pin 10
 * Spannungsversorgung UNO und NF-Verstärker: aus TRX (13.8V) via Spgregler 7805
 * 
 * Spricht auf Knopfdruck (Taster an Pin 8 des Arduino UNO) die aktuelle Frequenz des Transceivers vor
*/


#include <SoftwareSerial.h>
// the file FT857D.h has a lot of documentation

// Doku Talkie-Lib: https://github.com/ArminJo/Talkie
// Doku FT857D-Lib: 
#include "FT857D.h"     
FT857D radio;           // Instance of FT857D

#include <Arduino.h>
// Doku zur Lib in Talkie.h und den Beispielen sowie auf https://github.com/Phil-f6czv/Arduino-FT857-CAT
#include "Talkie.h"
#include "Vocab_MY.h"

// Schießt man einen NF-Verstärker an Pin3 des Arduino UNO an, dann gilt folgende Zeile
Talkie voice(true, false);        // Speech-Output at Arduino Pin 3 only

// Macht man sich es einfach und schließt einen kleinen Lautsprecher (ohne Verstärker)
// an Pin 3 und Pin 11 des Arduino UNO an, dann gilt diese Zeile
//Talkie voice;                   // Speech-Out at Pin 3 und Pin 11 inverted

//*****************************************************
// Variablen
//*****************************************************

unsigned long freq;
String sfreq;
  
float megahertz = 0;
uint16_t mega = 0;
String smega;

float hertz = 0;
String shertz;

void getFreq() 
{ 
  freq = radio.getFreqMode();
  sfreq = String(freq);

  // Megahertz
  megahertz = freq / 1E5;
  mega = abs(megahertz);
  smega = String(mega);     // Megahertz in smega

  // Hertz
  long temp = mega * 1E5;
  hertz = freq - temp;
  shertz = String(hertz,0);   // Hertz in shertz
}

void separate_chars()
{
  char c;
  // für Megahertz-Anteile
  for(int i=0; i< smega.length(); i++)
  {
    c = smega.charAt(i);
    speak_char(c);
  }

  voice.say(spmy_PAUSE1);
  voice.say(spmy_MEGA);
  voice.say(spmy_HERTZ);
  voice.say(spmy_PAUSE1);
  
  // für Hertz
  for(int ii=0; ii< shertz.length(); ii++)
  {
    c = shertz.charAt(ii);
    speak_char(c);
  }
  voice.say(spmy_HERTZ);
}

void speak_char(char c)
{
    switch (c)
    {
    case '1':
        voice.say(spmy_ONE);
        break;
    case '2':
        voice.say(spmy_TWO);
        break;
    case '3':
        voice.say(spmy_THREE);
        break;  
    case '4':
        voice.say(spmy_FOUR);
        break;  
    case '5':
        voice.say(spmy_FIVE);
        break;  
    case '6':
        voice.say(spmy_SIX);
        break;  
    case '7':
        voice.say(spmy_SEVEN);
        break;  
    case '8':
        voice.say(spmy_EIGHT);
        break;  
    case '9':
        voice.say(spmy_NINE);
        break;  
    case '0':
        voice.say(spmy_ZERO);
        break;  
    }
}

void work()
{
  getFreq();
  separate_chars();  
}


void setup() 
{
  radio.begin(9600);
  pinMode(3, OUTPUT);
  pinMode(8, INPUT_PULLUP);
}

void loop() 
{
  if(digitalRead(8)== 0)
  {
    work();
    radio.begin(9600);
  }
  delay(100);
}
